const {Command, flags} = require('@oclif/command')
const { prompt } = require('enquirer');
const ora = require('ora');
const chalk = require('chalk');
const execa = require('execa');
const path = require('path');
const copy = require('copy-template-dir');
const fs = require('fs'); 


const templateMap = {
	paria: 'https://github.com/Version-One-Studio/pariaTemplate.git', 
	balandra: 'https://github.com/jonotko/FashionTemplateWS.git',
	storebay: 'https://github.com/jonotko/StorebayTemplateWS.git'
}

class InitCommand extends Command {
  async run() {

		const {flags} = this.parse(InitCommand)
		
		// Check if shopName is specified
		if(typeof flags.shopName === 'undefined'){
				const response = await prompt({
					type: 'input',
					name: 'shopName',
					message: 'What is the name of your store? (This is the name part before myshopify.com)'
				})

				flags.shopName = response.shopName
		}

		// Check if apiKey is specified
		if (typeof flags.apiKey === 'undefined'){
			const response = await prompt({
				type: 'input',
				name: 'apiKey',
				message: `What is the API Key for ${chalk.cyan(flags.shopName)}`
			})

			flags.apiKey = response.apiKey
		}

		// Check if password is specified
		if(typeof flags.password === 'undefined'){
			const response = await prompt({
				type: 'password',
				name: 'password',
				message: `What is the password for ${chalk.cyan(flags.shopName)}`
			})

			flags.password = response.password;
		}

		// Check if storeFrontKey is specified
		if(typeof flags.storefrontAccessToken === 'undefined'){
			const response = await prompt({
				type: 'input',
				name: 'storefrontAccessToken',
				message: `What is the Storefront access token for ${chalk.cyan(flags.shopName)}`
			});

			flags.storefrontAccessToken = response.storefrontAccessToken;
		}

		// Check if template is specified
		if(typeof flags.template === 'undefined'){
			const response = await prompt({
				type: 'select',
				name: 'template',
				message: `What template would you like to use for ${chalk.cyan(flags.shopName)}?`,
				choices: Object.keys(templateMap)
			})

			flags.template = response.template;
		}

    this.log(`Shop Name: ${flags.shopName} from /Users/jonathan/Desktop/Clients/VersionOneStudio/wipaytemplate/src/commands/init.js`);
    this.log(`API Key: ${flags.apiKey} from /Users/jonathan/Desktop/Clients/VersionOneStudio/wipaytemplate/src/commands/init.js`);
    this.log(`Password ${'*'.repeat(flags.password.length)} from /Users/jonathan/Desktop/Clients/VersionOneStudio/wipaytemplate/src/commands/init.js`);
    this.log(`Storefront access token: ${flags.storefrontAccessToken} from /Users/jonathan/Desktop/Clients/VersionOneStudio/wipaytemplate/src/commands/init.js`);
		this.log(`Template: ${flags.template} from /Users/jonathan/Desktop/Clients/VersionOneStudio/wipaytemplate/src/commands/init.js`);
		
		let spinner = ora(`Grabbing your template 😁`).start();
		
		try {
			await execa('mkdir', [path.join(process.cwd(), '.temp')]);
			process.chdir('./.temp')
			const { stdout } = await execa('git', ['clone', templateMap[flags.template], 'newtemplate']);
			spinner.succeed();
			this.log(`${chalk.green('Template Retrieved')}`);
			process.chdir('..')
		}catch (error){
			spinner.fail();
			this.log(`${chalk.red(error)}`);
		}

		const vars = { 
			shopName: flags.shopName,
			apiKey: flags.apiKey,
			storefrontAccessToken: flags.storefrontAccessToken,
			password: flags.password
		}

		spinner = ora("Configuring your template.."). start();

		const storeName = flags.shopName;

		const inDir = path.join(process.cwd(), "./.temp/newtemplate")
		const outDir = path.join(process.cwd(), `${storeName}`)

		copy(inDir, outDir, vars, (err, createdFiles) => {
			if (err) throw err
			spinner.succeed();
		
			console.log(`${chalk.green('Done configuring template')}`)

			spinner = ora("Setting Environment").start();
			const inFile = path.join(__dirname, "../env");
			const outFile = path.join(process.cwd(), `${storeName}/`);
			
			copy(inFile, outFile, vars, async (err, createdFiles) => {
				if (err) throw err
				spinner.succeed();
				
				console.log('done!')

				copyImagesProperly(storeName);
			})

		})

	
  }
}

const copyImagesProperly = (storeName) => {

	// Construct template path
	const inDir = path.join(process.cwd(), "./.temp/newtemplate/src/images");

	let filesToCopy = fs.readdirSync(inDir).map(file => { 

		console.log(file); 
		// construct output path
		const outdir = path.join(process.cwd(), `${storeName}/src/images/${file}`);
		console.log(outdir);

		return new Promise((resolve, reject) => {

			fs.copyFile(`${inDir}/${file}`, outdir, err => {
				if (err) reject(err)
				resolve(chalk.green(`Copied: ${file} to ${outdir}`))
			})
		});

	}); 

	Promise.all(filesToCopy).then(values => {

		values.forEach( value => console.log(value));

		cleanup(storeName)
	})
	
}

const cleanup = async (storeName) => {
	const spinner = ora("Cleaning up").start()
	// remove temp folder
	await execa('rm', ['-rf', path.join(process.cwd(), '.temp')]);
	// remove git folder
	process.chdir(`./${storeName}`);
	await execa('rm', ['-rf', '.git']);
	spinner.succeed();
}

InitCommand.description = `Scaffolds a wipay ecommerce template with checkout
...
Extra documentation goes here
`

InitCommand.flags = {
	shopName: flags.string({char: 'sn', description: 'Name of your shopify store'}),
	apiKey: flags.string({char: 'ak', description: 'admin api key for private app'}),
	password: flags.string({char: 'pw', description: 'admin password for api key'}),
  storefrontAccessToken: flags.string({char: 'sft', description: 'Your shopify storefront access token'}),
  template: flags.string({char: 't', description: 'Ecommerce template to install'}),
}

module.exports = InitCommand
